* OWNJOB Set a job's owner
*
* pjwitte 2o22+
* All code is my own work except as and where stated
*
* V0.01, pjw, 2022 Jan 03
*
* Compatible with QDOS, Minerva and SMSQ/E
*

        section code

        xdef OWNJOB

        xref l2frtrd6

        include dev8_keys_qdos_sms
        include dev8_keys_sys
        include dev8_keys_sbasic
        include dev8_keys_jcbq
        include dev8_keys_qlv
        include dev8_keys_err


*
OWNJOB
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* er = OWNJOB(<target_job_ID>[, <owner_ID>])
*
* Set a job to be "owned" by another job, where
*
* er = return error code, eg
*        invalid job:    either of the IDs given is invalid, or target = 0
*        already exists: the job is already owned by another job than 0!
*        not complete:   cant own owner!
*        in use:         both jobs have the same ID
*
*      target_job_ID is the target job's ID in full or -1
*
*      owner_ID (optional, default is me) ID of proposed owner job or -1
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*                                                                         *
*       Use with extreme care! as this is not an "authorised" action!     *
*                                                                         *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

        move.w sb.gtlin,a2      get some longs
        jsr (a2)
        beq.s ok

        rts                     parameter errors are "hard"

err_bp
        moveq #err.ipar,d0
        rts

ok
        move.w d3,d0            make count long
        beq.s err_bp             0 pars is no good

        lsl.l #2,d0             tidy this many bytes from ri stack

        moveq #-1,d7            d7 = owner ID (assume me)

        subq.w #2,d3            two longs?
        bhi.s err_bp             no, too many
        bcs.s onep               no, one par

        move.l 4(a6,a1.l),d7     two pars => some other owner ID

onep
        move.l (a6,a1.l),d6     d6 = target ID safe copy

        add.l d0,sb_arthp(a6)   4 or 8 bytes off ri stack

        tst.l d6
        beq.s err_nj            target cannot be 0!

        moveq #sms.info,d0      get job info and system variables

        moveq #-1,d1            "me"
        cmp.l d1,d7             me?
        bne.s target             no, assume owner ID specified. Try target

        trap #1                 yes, convert to real ID

        move.l d1,d7            d7 = proper owner ID
        bra.s skip              assume target ID properly specified..

target
        cmp.l d6,d1             is target "me"?
        bne.s skip2              no, but we still need -> sys vars

        trap #1                 yes, convert to proper ID

        move.l d1,d6            d6 = properly specified target ID
        bra.s skip              now for some more checks..

skip2
        trap #1                 get info

*
skip
        movea.l a0,a4           a4 -> sys vars

        cmp.l d6,d7
        beq.s err_iu            IDs cant be the same!

        move.l d6,d1            target..
        bsr.s chkjob             check validity and get owner

        tst.l d2                present target owner must(?) be 0
        bne.s err_ex             wasnt..

        move.l d7,d1            owner..
        bsr.s chkjob             check validity and get owner's owner

        cmp.l d2,d6             we cant own owner!!
        beq.s err_nc

        move.l d6,d1            we want target jcb
        lsl.w #2,d6             convert id to offset
        swap d1                 reach tag

        trap #0                 freeze Qdos

        movea.l sys_jbtb(a4),a0 get job table base
        movea.l 0(a0,d6.w),a0   a0 -> base of target jcb
        cmp.w jcb_tag(a0),d1    tag match?
        bne.s nojob              no, job was removed

        move.l d7,jcb_ownr(a0)   yes, set new owner
        moveq #0,d6             no error
        bra.s exsv              go to exit supervisor mode

*
* Check job ID valid and get its owner
*
chkjob
        moveq #sms.injb,d0      get job info
        moveq #0,d2             set top job
        trap #1
        tst.l d0                invalid ID?
        beq.s rts                no, good

        addq.l #4,sp             woops! (user error or job gone)
        bra.s err_ret           dont come back from this
rts
        rts
***

*
nojob
        moveq #err.ijob,d6      still in supervisor mode..

exsv
        andi.w #$d8ff,sr        unfreeze!

*
return
        bra l2frtrd6            make room, float error in d6 and return

err_nj
        moveq #err.ijob,d6      bad/no job
        bra.s return

err_nc
        moveq #err.nc,d6        not complete (cant own owner)
        bra.s return

err_iu
        moveq #err.fdiu,d6      in use (IDs same)
        bra.s return

err_ex
        moveq #err.fex,d0       already exists (other owner <> 0)

err_ret
        move.l d0,d6            unspecified error
        bra.s return

*
        end
